@@ -1,5 +1,5 @@
 use criterion::{criterion_group, criterion_main, Criterion};
-use deb822_lossless::lossless::Deb822;
+use deb822_lossless::Deb822;
 
 fn parse_deb822_benchmark(c: &mut Criterion) {
     let control_data =
