@@ -1,6 +1,9 @@
 use criterion::{criterion_group, criterion_main, Criterion};
-use deb822_lossless::lossy::Deb822;
 
+#[cfg(feature = "deb822-fast")]
+use deb822_fast::Deb822;
+
+#[cfg(feature = "deb822-fast")]
 fn parse_deb822_benchmark(c: &mut Criterion) {
     let control_data =
         std::fs::read_to_string(concat!(env!("CARGO_MANIFEST_DIR"), "/bench/Sources"))
@@ -13,5 +16,10 @@ fn parse_deb822_benchmark(c: &mut Criterion) {
     });
 }
 
+#[cfg(not(feature = "deb822-fast"))]
+fn parse_deb822_benchmark(_c: &mut Criterion) {
+    // Skip benchmark when deb822-fast feature is not enabled
+}
+
 criterion_group!(benches, parse_deb822_benchmark);
 criterion_main!(benches);
