@@ -42,9 +42,28 @@ use rowan::ast::AstNode;
 use std::path::Path;
 use std::str::FromStr;
 
+/// A positioned parse error containing location information.
+#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+pub struct PositionedParseError {
+    /// The error message
+    pub message: String,
+    /// The text range where the error occurred
+    pub range: rowan::TextRange,
+    /// Optional error code for categorization
+    pub code: Option<String>,
+}
+
+impl std::fmt::Display for PositionedParseError {
+    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
+        write!(f, "{}", self.message)
+    }
+}
+
+impl std::error::Error for PositionedParseError {}
+
 /// List of encountered syntax errors.
 #[derive(Debug, Clone, PartialEq, Eq, Hash)]
-pub struct ParseError(Vec<String>);
+pub struct ParseError(pub Vec<String>);
 
 impl std::fmt::Display for ParseError {
     fn fmt(&self, f: &mut std::fmt::Formatter) -> std::fmt::Result {
@@ -116,25 +135,32 @@ use rowan::GreenNodeBuilder;
 
 /// The parse results are stored as a "green tree".
 /// We'll discuss working with the results later
-struct Parse {
-    green_node: GreenNode,
+pub(crate) struct Parse {
+    pub(crate) green_node: GreenNode,
     #[allow(unused)]
-    errors: Vec<String>,
+    pub(crate) errors: Vec<String>,
+    pub(crate) positioned_errors: Vec<PositionedParseError>,
 }
 
-fn parse(text: &str) -> Parse {
-    struct Parser {
+pub(crate) fn parse(text: &str) -> Parse {
+    struct Parser<'a> {
         /// input tokens, including whitespace,
         /// in *reverse* order.
-        tokens: Vec<(SyntaxKind, String)>,
+        tokens: Vec<(SyntaxKind, &'a str)>,
         /// the in-progress tree.
         builder: GreenNodeBuilder<'static>,
         /// the list of syntax errors we've accumulated
         /// so far.
         errors: Vec<String>,
+        /// positioned errors with location information
+        positioned_errors: Vec<PositionedParseError>,
+        /// All tokens with their positions in forward order for position tracking
+        token_positions: Vec<(SyntaxKind, rowan::TextSize, rowan::TextSize)>,
+        /// current token index (counting from the end since tokens are in reverse)
+        current_token_index: usize,
     }
 
-    impl Parser {
+    impl<'a> Parser<'a> {
         fn parse_entry(&mut self) {
             while self.current() == Some(COMMENT) {
                 self.bump();
@@ -148,8 +174,11 @@ fn parse(text: &str) -> Parse {
                     }
                     Some(g) => {
                         self.builder.start_node(ERROR.into());
+                        self.add_positioned_error(
+                            format!("expected newline, got {g:?}"),
+                            Some("unexpected_token".to_string()),
+                        );
                         self.bump();
-                        self.errors.push(format!("expected newline, got {:?}", g));
                         self.builder.finish_node();
                     }
                 }
@@ -163,10 +192,13 @@ fn parse(text: &str) -> Parse {
                 self.skip_ws();
             } else {
                 self.builder.start_node(ERROR.into());
+                self.add_positioned_error(
+                    "expected key".to_string(),
+                    Some("missing_key".to_string()),
+                );
                 if self.current().is_some() {
                     self.bump();
                 }
-                self.errors.push("expected key".to_string());
                 self.builder.finish_node();
             }
             if self.current() == Some(COLON) {
@@ -174,11 +206,13 @@ fn parse(text: &str) -> Parse {
                 self.skip_ws();
             } else {
                 self.builder.start_node(ERROR.into());
+                self.add_positioned_error(
+                    format!("expected ':', got {:?}", self.current()),
+                    Some("missing_colon".to_string()),
+                );
                 if self.current().is_some() {
                     self.bump();
                 }
-                self.errors
-                    .push(format!("expected ':', got {:?}", self.current()));
                 self.builder.finish_node();
             }
             loop {
@@ -195,8 +229,11 @@ fn parse(text: &str) -> Parse {
                     }
                     Some(g) => {
                         self.builder.start_node(ERROR.into());
+                        self.add_positioned_error(
+                            format!("expected newline, got {g:?}"),
+                            Some("unexpected_token".to_string()),
+                        );
                         self.bump();
-                        self.errors.push(format!("expected newline, got {:?}", g));
                         self.builder.finish_node();
                     }
                 }
@@ -236,17 +273,44 @@ fn parse(text: &str) -> Parse {
             Parse {
                 green_node: self.builder.finish(),
                 errors: self.errors,
+                positioned_errors: self.positioned_errors,
             }
         }
         /// Advance one token, adding it to the current branch of the tree builder.
         fn bump(&mut self) {
             let (kind, text) = self.tokens.pop().unwrap();
-            self.builder.token(kind.into(), text.as_str());
+            self.builder.token(kind.into(), text);
+            if self.current_token_index > 0 {
+                self.current_token_index -= 1;
+            }
         }
         /// Peek at the first unprocessed token
         fn current(&self) -> Option<SyntaxKind> {
             self.tokens.last().map(|(kind, _)| *kind)
         }
+
+        /// Add a positioned error at the current position
+        fn add_positioned_error(&mut self, message: String, code: Option<String>) {
+            let range = if self.current_token_index < self.token_positions.len() {
+                let (_, start, end) = self.token_positions[self.current_token_index];
+                rowan::TextRange::new(start, end)
+            } else {
+                // Default to end of text if no current token
+                let end = self
+                    .token_positions
+                    .last()
+                    .map(|(_, _, end)| *end)
+                    .unwrap_or_else(|| rowan::TextSize::from(0));
+                rowan::TextRange::new(end, end)
+            };
+
+            self.positioned_errors.push(PositionedParseError {
+                message: message.clone(),
+                range,
+                code,
+            });
+            self.errors.push(message);
+        }
         fn skip_ws(&mut self) {
             while self.current() == Some(WHITESPACE) || self.current() == Some(COMMENT) {
                 self.bump()
@@ -269,14 +333,29 @@ fn parse(text: &str) -> Parse {
         }
     }
 
-    let mut tokens = lex(text)
-        .map(|(k, t)| (k, t.to_string()))
-        .collect::<Vec<_>>();
+    let mut tokens = lex(text).collect::<Vec<_>>();
+
+    // Build token positions in forward order
+    let mut token_positions = Vec::new();
+    let mut position = rowan::TextSize::from(0);
+    for (kind, text) in &tokens {
+        let start = position;
+        let end = start + rowan::TextSize::of(*text);
+        token_positions.push((*kind, start, end));
+        position = end;
+    }
+
+    // Reverse tokens for parsing (but keep positions in forward order)
     tokens.reverse();
+    let current_token_index = tokens.len().saturating_sub(1);
+
     Parser {
         tokens,
         builder: GreenNodeBuilder::new(),
         errors: Vec::new(),
+        positioned_errors: Vec::new(),
+        token_positions,
+        current_token_index,
     }
     .parse()
 }
@@ -306,8 +385,10 @@ impl Parse {
 
 macro_rules! ast_node {
     ($ast:ident, $kind:ident) => {
-        /// An AST node representing a $ast.
-        #[derive(PartialEq, Eq, Hash)]
+        #[doc = "An AST node representing a `"]
+        #[doc = stringify!($ast)]
+        #[doc = "`."]
+        #[derive(Debug, Clone, PartialEq, Eq, Hash)]
         #[repr(transparent)]
         pub struct $ast(SyntaxNode);
         impl $ast {
@@ -345,12 +426,6 @@ macro_rules! ast_node {
     };
 }
 
-impl std::fmt::Debug for Deb822 {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-        f.debug_struct("Deb822").finish()
-    }
-}
-
 ast_node!(Deb822, ROOT);
 ast_node!(Paragraph, PARAGRAPH);
 ast_node!(Entry, ENTRY);
@@ -371,6 +446,11 @@ impl Deb822 {
         Deb822(SyntaxNode::new_root_mut(builder.finish()))
     }
 
+    /// Parse deb822 text, returning a Parse result
+    pub fn parse(text: &str) -> crate::Parse<Deb822> {
+        crate::Parse::parse_deb822(text)
+    }
+
     /// Provide a formatter that can handle indentation and trailing separators
     ///
     /// # Arguments
@@ -457,8 +537,41 @@ impl Deb822 {
         self.0.children().filter_map(Paragraph::cast)
     }
 
-    /// Add a new empty paragraph to the end of the file.
-    pub fn add_paragraph(&mut self) -> Paragraph {
+    /// Converts the perceptual paragraph index to the node index.
+    fn convert_index(&self, index: usize) -> Option<usize> {
+        let mut current_pos = 0usize;
+        if index == 0 {
+            return Some(0);
+        }
+        for (i, node) in self.0.children_with_tokens().enumerate() {
+            if node.kind() == PARAGRAPH {
+                if current_pos == index {
+                    return Some(i);
+                }
+                current_pos += 1;
+            }
+        }
+
+        None
+    }
+
+    /// Delete trailing empty lines after specified node and before any non-empty line nodes.
+    fn delete_trailing_space(&self, start: usize) {
+        for (i, node) in self.0.children_with_tokens().enumerate() {
+            if i < start {
+                continue;
+            }
+            if node.kind() != EMPTY_LINE {
+                return;
+            }
+            // this is not a typo, the index will shift by one after deleting the node
+            // so instead of deleting using `i`, we use `start` as the start index
+            self.0.splice_children(start..start + 1, []);
+        }
+    }
+
+    /// Shared internal function to insert a new paragraph into the file.
+    fn insert_empty_paragraph(&mut self, index: Option<usize>) -> Paragraph {
         let paragraph = Paragraph::new();
         let mut to_insert = vec![];
         if self.0.children().count() > 0 {
@@ -469,13 +582,72 @@ impl Deb822 {
             to_insert.push(SyntaxNode::new_root_mut(builder.finish()).into());
         }
         to_insert.push(paragraph.0.clone().into());
-        self.0.splice_children(
-            self.0.children().count()..self.0.children().count(),
-            to_insert,
-        );
+        let insertion_point = match index {
+            Some(i) => {
+                if to_insert.len() > 1 {
+                    to_insert.swap(0, 1);
+                }
+                i
+            }
+            None => self.0.children().count(),
+        };
+        self.0
+            .splice_children(insertion_point..insertion_point, to_insert);
         paragraph
     }
 
+    /// Insert a new empty paragraph into the file after specified index.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use deb822_lossless::{Deb822, Paragraph};
+    /// let mut d: Deb822 = vec![
+    ///     vec![("Foo", "Bar"), ("Baz", "Qux")].into_iter().collect(),
+    ///     vec![("A", "B"), ("C", "D")].into_iter().collect(),
+    /// ]
+    /// .into_iter()
+    /// .collect();
+    /// let mut p = d.insert_paragraph(0);
+    /// p.set("Foo", "Baz");
+    /// assert_eq!(d.to_string(), "Foo: Baz\n\nFoo: Bar\nBaz: Qux\n\nA: B\nC: D\n");
+    /// let mut another = d.insert_paragraph(1);
+    /// another.set("Y", "Z");
+    /// assert_eq!(d.to_string(), "Foo: Baz\n\nY: Z\n\nFoo: Bar\nBaz: Qux\n\nA: B\nC: D\n");
+    /// ```
+    pub fn insert_paragraph(&mut self, index: usize) -> Paragraph {
+        self.insert_empty_paragraph(self.convert_index(index))
+    }
+
+    /// Remove the paragraph at the specified index from the file.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use deb822_lossless::Deb822;
+    /// let mut d: Deb822 = vec![
+    ///     vec![("Foo", "Bar"), ("Baz", "Qux")].into_iter().collect(),
+    ///     vec![("A", "B"), ("C", "D")].into_iter().collect(),
+    /// ]
+    /// .into_iter()
+    /// .collect();
+    /// d.remove_paragraph(0);
+    /// assert_eq!(d.to_string(), "A: B\nC: D\n");
+    /// d.remove_paragraph(0);
+    /// assert_eq!(d.to_string(), "");
+    /// ```
+    pub fn remove_paragraph(&mut self, index: usize) {
+        if let Some(index) = self.convert_index(index) {
+            self.0.splice_children(index..index + 1, []);
+            self.delete_trailing_space(index);
+        }
+    }
+
+    /// Add a new empty paragraph to the end of the file.
+    pub fn add_paragraph(&mut self) -> Paragraph {
+        self.insert_empty_paragraph(None)
+    }
+
     /// Read a deb822 file from the given path.
     pub fn from_file(path: impl AsRef<Path>) -> Result<Self, Error> {
         let text = std::fs::read_to_string(path)?;
@@ -697,7 +869,7 @@ impl Paragraph {
     }
 
     /// Returns an iterator over all entries in the paragraph.
-    fn entries(&self) -> impl Iterator<Item = Entry> + '_ {
+    pub fn entries(&self) -> impl Iterator<Item = Entry> + '_ {
         self.0.children().filter_map(Entry::cast)
     }
 
@@ -710,7 +882,7 @@ impl Paragraph {
     /// Returns an iterator over all values for the given key in the paragraph.
     pub fn get_all<'a>(&'a self, key: &'a str) -> impl Iterator<Item = String> + 'a {
         self.items()
-            .filter_map(move |(k, v)| if k.as_str() == key { Some(v) } else { None })
+            .filter_map(move |(k, v)| if k == key { Some(v) } else { None })
     }
 
     /// Returns an iterator over all keys in the paragraph.
@@ -788,16 +960,38 @@ impl std::str::FromStr for Paragraph {
 }
 
 #[cfg(feature = "python-debian")]
-impl pyo3::ToPyObject for Paragraph {
-    fn to_object(&self, py: pyo3::Python) -> pyo3::PyObject {
+impl<'py> pyo3::IntoPyObject<'py> for Paragraph {
+    type Target = pyo3::PyAny;
+    type Output = pyo3::Bound<'py, Self::Target>;
+    type Error = pyo3::PyErr;
+
+    fn into_pyobject(self, py: pyo3::Python<'py>) -> Result<Self::Output, Self::Error> {
         use pyo3::prelude::*;
-        let d = pyo3::types::PyDict::new_bound(py);
+        let d = pyo3::types::PyDict::new(py);
         for (k, v) in self.items() {
-            d.set_item(k, v).unwrap();
+            d.set_item(k, v)?;
         }
-        let m = py.import_bound("debian.deb822").unwrap();
-        let cls = m.getattr("Deb822").unwrap();
-        cls.call1((d,)).unwrap().to_object(py)
+        let m = py.import("debian.deb822")?;
+        let cls = m.getattr("Deb822")?;
+        cls.call1((d,))
+    }
+}
+
+#[cfg(feature = "python-debian")]
+impl<'a, 'py> pyo3::IntoPyObject<'py> for &'a Paragraph {
+    type Target = pyo3::PyAny;
+    type Output = pyo3::Bound<'py, Self::Target>;
+    type Error = pyo3::PyErr;
+
+    fn into_pyobject(self, py: pyo3::Python<'py>) -> Result<Self::Output, Self::Error> {
+        use pyo3::prelude::*;
+        let d = pyo3::types::PyDict::new(py);
+        for (k, v) in self.items() {
+            d.set_item(k, v)?;
+        }
+        let m = py.import("debian.deb822")?;
+        let cls = m.getattr("Deb822")?;
+        cls.call1((d,))
     }
 }
 
@@ -812,6 +1006,62 @@ impl pyo3::FromPyObject<'_> for Paragraph {
 }
 
 impl Entry {
+    /// Returns the text range of this entry in the source text.
+    pub fn text_range(&self) -> rowan::TextRange {
+        self.0.text_range()
+    }
+
+    /// Returns the text range of the key (field name) in this entry.
+    pub fn key_range(&self) -> Option<rowan::TextRange> {
+        self.0
+            .children_with_tokens()
+            .filter_map(|it| it.into_token())
+            .find(|it| it.kind() == KEY)
+            .map(|it| it.text_range())
+    }
+
+    /// Returns the text range of the colon separator in this entry.
+    pub fn colon_range(&self) -> Option<rowan::TextRange> {
+        self.0
+            .children_with_tokens()
+            .filter_map(|it| it.into_token())
+            .find(|it| it.kind() == COLON)
+            .map(|it| it.text_range())
+    }
+
+    /// Returns the text range of the value portion (excluding the key and colon) in this entry.
+    /// This includes all VALUE tokens and any continuation lines.
+    pub fn value_range(&self) -> Option<rowan::TextRange> {
+        let value_tokens: Vec<_> = self
+            .0
+            .children_with_tokens()
+            .filter_map(|it| it.into_token())
+            .filter(|it| it.kind() == VALUE)
+            .collect();
+
+        if value_tokens.is_empty() {
+            return None;
+        }
+
+        let first = value_tokens.first().unwrap();
+        let last = value_tokens.last().unwrap();
+        Some(rowan::TextRange::new(
+            first.text_range().start(),
+            last.text_range().end(),
+        ))
+    }
+
+    /// Returns the text ranges of all individual value lines in this entry.
+    /// Multi-line values will return multiple ranges.
+    pub fn value_line_ranges(&self) -> Vec<rowan::TextRange> {
+        self.0
+            .children_with_tokens()
+            .filter_map(|it| it.into_token())
+            .filter(|it| it.kind() == VALUE)
+            .map(|it| it.text_range())
+            .collect()
+    }
+
     /// Create a new entry with the given key and value.
     pub fn new(key: &str, value: &str) -> Entry {
         let mut builder = GreenNodeBuilder::new();
@@ -948,13 +1198,24 @@ impl Entry {
 
     /// Returns the value of the entry.
     pub fn value(&self) -> String {
-        self.0
+        let mut parts = self
+            .0
             .children_with_tokens()
             .filter_map(|it| it.into_token())
             .filter(|it| it.kind() == VALUE)
-            .map(|it| it.text().to_string())
-            .collect::<Vec<_>>()
-            .join("\n")
+            .map(|it| it.text().to_string());
+
+        match parts.next() {
+            None => String::new(),
+            Some(first) => {
+                let mut result = first;
+                for part in parts {
+                    result.push('\n');
+                    result.push_str(&part);
+                }
+                result
+            }
+        }
     }
 
     /// Detach this entry from the paragraph.
@@ -967,12 +1228,7 @@ impl FromStr for Deb822 {
     type Err = ParseError;
 
     fn from_str(s: &str) -> Result<Self, Self::Err> {
-        let parsed = parse(s);
-        if parsed.errors.is_empty() {
-            Ok(parsed.root_mut())
-        } else {
-            Err(ParseError(parsed.errors))
-        }
+        Deb822::parse(s).to_result()
     }
 }
 
@@ -1189,16 +1445,20 @@ fn rebuild_value(
             builder.token(WHITESPACE.into(), " ");
         }
         // Strip leading whitespace and newlines
-        while let Some((k, _t)) = tokens.first() {
-            if *k == NEWLINE || *k == WHITESPACE {
-                tokens.remove(0);
+        let mut start_idx = 0;
+        while start_idx < tokens.len() {
+            if tokens[start_idx].0 == NEWLINE || tokens[start_idx].0 == WHITESPACE {
+                start_idx += 1;
             } else {
                 break;
             }
         }
+        tokens.drain(..start_idx);
+        // Pre-allocate indentation string to avoid repeated allocations
+        let indent_str = " ".repeat(indentation as usize);
         for (k, t) in tokens {
             if last_was_newline {
-                builder.token(INDENT.into(), &" ".repeat(indentation as usize));
+                builder.token(INDENT.into(), &indent_str);
             }
             builder.token(k.into(), &t);
             last_was_newline = k == NEWLINE;
@@ -1294,7 +1554,7 @@ Description: This is a description
         let mut p = ps.next().unwrap();
         p.set("Foo", "Bar");
         p.remove("Section");
-        p.remove("Nonexistant");
+        p.remove("Nonexistent");
         assert_eq!(p.get("Foo").as_deref(), Some("Bar"));
         assert_eq!(
             p.to_string(),
@@ -1405,6 +1665,43 @@ Foo: Blah
         );
     }
 
+    #[test]
+    fn test_crud_paragraph() {
+        let mut d = super::Deb822::new();
+        let mut p = d.insert_paragraph(0);
+        p.set("Foo", "Bar");
+        assert_eq!(p.get("Foo").as_deref(), Some("Bar"));
+        assert_eq!(
+            d.to_string(),
+            r#"Foo: Bar
+"#
+        );
+
+        // test prepend
+        let mut p = d.insert_paragraph(0);
+        p.set("Foo", "Blah");
+        assert_eq!(p.get("Foo").as_deref(), Some("Blah"));
+        assert_eq!(
+            d.to_string(),
+            r#"Foo: Blah
+
+Foo: Bar
+"#
+        );
+
+        // test delete
+        d.remove_paragraph(1);
+        assert_eq!(d.to_string(), "Foo: Blah\n\n");
+
+        // test update again
+        p.set("Foo", "Baz");
+        assert_eq!(d.to_string(), "Foo: Baz\n\n");
+
+        // test delete again
+        d.remove_paragraph(0);
+        assert_eq!(d.to_string(), "");
+    }
+
     #[test]
     fn test_multiline_entry() {
         use super::SyntaxKind::*;
@@ -1627,6 +1924,19 @@ C: D
         assert_eq!(ParseError(vec!["foo".to_string()]).to_string(), "foo\n");
     }
 
+    #[test]
+    fn test_positioned_parse_error() {
+        let error = PositionedParseError {
+            message: "test error".to_string(),
+            range: rowan::TextRange::new(rowan::TextSize::from(5), rowan::TextSize::from(10)),
+            code: Some("test_code".to_string()),
+        };
+        assert_eq!(error.to_string(), "test error");
+        assert_eq!(error.range.start(), rowan::TextSize::from(5));
+        assert_eq!(error.range.end(), rowan::TextSize::from(10));
+        assert_eq!(error.code, Some("test_code".to_string()));
+    }
+
     #[test]
     fn test_format_error() {
         assert_eq!(
@@ -1648,4 +1958,150 @@ Maintainer: Bar Foo <bar@example.com>"#
             vec!["Foo Bar <foo@example.com>", "Bar Foo <bar@example.com>"]
         );
     }
+
+    #[test]
+    fn test_entry_ranges() {
+        let input = r#"Package: test-package
+Maintainer: Test User <test@example.com>
+Description: A simple test package
+ with multiple lines
+ of description text"#;
+
+        let deb822 = super::Deb822::from_str(input).unwrap();
+        let paragraph = deb822.paragraphs().next().unwrap();
+        let entries: Vec<_> = paragraph.entries().collect();
+
+        // Test first entry (Package)
+        let package_entry = &entries[0];
+        assert_eq!(package_entry.key(), Some("Package".to_string()));
+
+        // Test key_range
+        let key_range = package_entry.key_range().unwrap();
+        assert_eq!(
+            &input[key_range.start().into()..key_range.end().into()],
+            "Package"
+        );
+
+        // Test colon_range
+        let colon_range = package_entry.colon_range().unwrap();
+        assert_eq!(
+            &input[colon_range.start().into()..colon_range.end().into()],
+            ":"
+        );
+
+        // Test value_range
+        let value_range = package_entry.value_range().unwrap();
+        assert_eq!(
+            &input[value_range.start().into()..value_range.end().into()],
+            "test-package"
+        );
+
+        // Test text_range covers the whole entry
+        let text_range = package_entry.text_range();
+        assert_eq!(
+            &input[text_range.start().into()..text_range.end().into()],
+            "Package: test-package\n"
+        );
+
+        // Test single-line value_line_ranges
+        let value_lines = package_entry.value_line_ranges();
+        assert_eq!(value_lines.len(), 1);
+        assert_eq!(
+            &input[value_lines[0].start().into()..value_lines[0].end().into()],
+            "test-package"
+        );
+    }
+
+    #[test]
+    fn test_multiline_entry_ranges() {
+        let input = r#"Description: Short description
+ Extended description line 1
+ Extended description line 2"#;
+
+        let deb822 = super::Deb822::from_str(input).unwrap();
+        let paragraph = deb822.paragraphs().next().unwrap();
+        let entry = paragraph.entries().next().unwrap();
+
+        assert_eq!(entry.key(), Some("Description".to_string()));
+
+        // Test value_range spans all lines
+        let value_range = entry.value_range().unwrap();
+        let full_value = &input[value_range.start().into()..value_range.end().into()];
+        assert!(full_value.contains("Short description"));
+        assert!(full_value.contains("Extended description line 1"));
+        assert!(full_value.contains("Extended description line 2"));
+
+        // Test value_line_ranges gives individual lines
+        let value_lines = entry.value_line_ranges();
+        assert_eq!(value_lines.len(), 3);
+
+        assert_eq!(
+            &input[value_lines[0].start().into()..value_lines[0].end().into()],
+            "Short description"
+        );
+        assert_eq!(
+            &input[value_lines[1].start().into()..value_lines[1].end().into()],
+            "Extended description line 1"
+        );
+        assert_eq!(
+            &input[value_lines[2].start().into()..value_lines[2].end().into()],
+            "Extended description line 2"
+        );
+    }
+
+    #[test]
+    fn test_entries_public_access() {
+        let input = r#"Package: test
+Version: 1.0"#;
+
+        let deb822 = super::Deb822::from_str(input).unwrap();
+        let paragraph = deb822.paragraphs().next().unwrap();
+
+        // Test that entries() method is now public
+        let entries: Vec<_> = paragraph.entries().collect();
+        assert_eq!(entries.len(), 2);
+        assert_eq!(entries[0].key(), Some("Package".to_string()));
+        assert_eq!(entries[1].key(), Some("Version".to_string()));
+    }
+
+    #[test]
+    fn test_empty_value_ranges() {
+        let input = r#"EmptyField: "#;
+
+        let deb822 = super::Deb822::from_str(input).unwrap();
+        let paragraph = deb822.paragraphs().next().unwrap();
+        let entry = paragraph.entries().next().unwrap();
+
+        assert_eq!(entry.key(), Some("EmptyField".to_string()));
+
+        // Empty value should still have ranges
+        assert!(entry.key_range().is_some());
+        assert!(entry.colon_range().is_some());
+
+        // Empty value might not have value tokens
+        let value_lines = entry.value_line_ranges();
+        // This depends on how the parser handles empty values
+        // but we should not panic
+        assert!(value_lines.len() <= 1);
+    }
+
+    #[test]
+    fn test_range_ordering() {
+        let input = r#"Field: value"#;
+
+        let deb822 = super::Deb822::from_str(input).unwrap();
+        let paragraph = deb822.paragraphs().next().unwrap();
+        let entry = paragraph.entries().next().unwrap();
+
+        let key_range = entry.key_range().unwrap();
+        let colon_range = entry.colon_range().unwrap();
+        let value_range = entry.value_range().unwrap();
+        let text_range = entry.text_range();
+
+        // Verify ranges are in correct order
+        assert!(key_range.end() <= colon_range.start());
+        assert!(colon_range.end() <= value_range.start());
+        assert!(key_range.start() >= text_range.start());
+        assert!(value_range.end() <= text_range.end());
+    }
 }
